@@ -491,6 +491,23 @@ def estimate_lmin(G, maxiter=2000):
     return sorted(lmins)[0]
 
 
+def estimate_lmax(G):
+    try:
+        lmax = sparse.linalg.eigsh(G.L, k=1, tol=5e-3,
+                                   ncv=min(G.N, 10),
+                                   return_eigenvectors=False)
+        lmax = lmax[0]
+        lmax *= 1.01  # Increase by 1 percent to be robust to errors.
+        upper_bound = G._get_upper_bound()
+        if lmax > upper_bound:
+            lmax = upper_bound
+    except sparse.linalg.ArpackNoConvergence:
+        G.logger.warning('Lanczos method did not converge. '
+                            'Using an alternative method.')
+        lmax = G._get_upper_bound()
+    return lmax
+
+
 class LGraphFourier(gsp.graphs.fourier.GraphFourier):
     def compute_fourier_basis(self, recompute=False, spectrum_only=False):
         if hasattr(self, '_e') and self._e is not None and hasattr(self, '_U') and self._U is not None and not recompute:
@@ -612,18 +629,24 @@ class Hypergraph(LGraph):
 
 class BigGraph(LGraphFourier, gsp.graphs.Graph):
     @staticmethod
-    def create_from(G):
+    def create_from(G, s=1, q=0):
         coords = G.coords if hasattr(G, 'coords') else None
-        return BigGraph(G.W, lap_type=G.lap_type, coords=coords, plotting=G.plotting)
+        return BigGraph(G.W, lap_type=G.lap_type, s=s, q=q, coords=coords, plotting=G.plotting)
 
-    def __init__(self, W, lap_type='combinatorial', q=0, coords=None, plotting={}):
+    def __init__(self, W, lap_type='combinatorial', s=1, q=0, coords=None, plotting={}):
+        """
+        s = deformation parameter (1 - sA + s^2(D - 1))
+        q = magnetic parameter [0, 1]
+        """
         self.lap_type = None
         self._I = None
+        self.s = s
         self.q = q
         self._Iq = None
         self._Wq = None
         self._dwq = None
         self._lmin = None
+        self._lmax = None
         self._n_connected = None
         self._graph_init(W, lap_type=lap_type, coords=coords, plotting=plotting)
 
@@ -700,8 +723,7 @@ class BigGraph(LGraphFourier, gsp.graphs.Graph):
             # Those attributes are invalidated when the Laplacian is changed.
             # Alternative: don't allow the user to change the Laplacian.
             self._lmin = None
-            if hasattr(self, '_lmax'):
-                del self._lmax
+            self._lmax = None
             if hasattr(self, '_U'):
                 del self._U
             if hasattr(self, '_e'):
@@ -715,13 +737,26 @@ class BigGraph(LGraphFourier, gsp.graphs.Graph):
 
         if lap_type == 'combinatorial':
             D = sparse.diags(self.dwq)
-            self.L = D - self.Wq
+            if self.s == 1:
+                self.L = D - self.Wq
+            elif self.s == -1:
+                self.L = D + self.Wq
+            else:
+                I = sparse.identity(self.N)
+                self.L = I - self.s * self.Wq + self.s * self.s * (D - I)
         elif lap_type == 'normalized':
-            d = np.zeros(self.N)
+            d_neg_sqrt = np.zeros(self.N)
             disconnected = (self.dwq == 0)
-            np.power(self.dwq, -0.5, where=~disconnected, out=d)
-            D = sparse.diags(d)
-            self.L = sparse.identity(self.N) - D * self.Wq * D
+            np.power(self.dwq, -0.5, where=~disconnected, out=d_neg_sqrt)
+            D_neg_sqrt = sparse.diags(d_neg_sqrt)
+            if self.s == 1:
+                self.L = sparse.identity(self.N) - D_neg_sqrt * self.Wq * D_neg_sqrt
+            elif self.s == -1:
+                self.L = sparse.identity(self.N) + D_neg_sqrt * self.Wq * D_neg_sqrt
+            else:
+                I = sparse.identity(self.N)
+                D = sparse.diags(self.dwq)
+                self.L = D_neg_sqrt @ (I - self.s * self.Wq + self.s * self.s * (D - I)) @ D_neg_sqrt
             self.L[disconnected, disconnected] = 0
             self.L.eliminate_zeros()
         else:
@@ -810,6 +845,21 @@ class BigGraph(LGraphFourier, gsp.graphs.Graph):
         if self._lmin is None:
             self._lmin = estimate_lmin(self)
 
+    @property
+    def lmax(self):
+        if self._lmax is None:
+            self.logger.warning('The largest eigenvalue G.lmax is not '
+                                'available, we need to estimate it. '
+                                'Explicitly call G.estimate_lmax() or '
+                                'G.compute_fourier_basis() '
+                                'once beforehand to suppress the warning.')
+            self.estimate_lmax()
+        return self._lmax
+
+    def estimate_lmax(self):
+        if self._lmax is None:
+            self._lmax = estimate_lmax(self)
+
     def count_components(self):
         if self._n_connected is None:
             self._n_connected = count_components(self)
@@ -822,9 +872,10 @@ class BigGraph(LGraphFourier, gsp.graphs.Graph):
 
 
 class BipartiteGraph(BigGraph):
-    def __init__(self, W, lap_type='combinatorial', q=0, coords=None, plotting={}):
-        super().__init__(W, lap_type=lap_type, q=q, coords=coords, plotting=plotting)
-        self._lmax = 2
+    def __init__(self, W, lap_type='combinatorial', s=1, q=0, coords=None, plotting={}):
+        super().__init__(W, lap_type=lap_type, s=s, q=q, coords=coords, plotting=plotting)
+        if self.lap_type == 'normalized':
+            self._lmax = 2
 
     def compute_fourier_basis(self, recompute=False):
         LGraphFourier.compute_fourier_basis(self, recompute)
